"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const http = require("http");
const https = require("https");
const url_1 = require("url");
class ServerAddress {
    constructor(app) {
        this.isListening = false;
        this.connections = new Set();
        this.server = typeof app === "function" ? http.createServer(app) : app;
        this.protocol = app instanceof https.Server ? "https" : "http";
    }
    /**
     * Listen to a random port number.
     */
    listen(cb) {
        if (!this.server.address()) {
            this.server.listen(0, cb);
        }
        else if (cb) {
            process.nextTick(cb);
        }
        if (!this.isListening) {
            this.isListening = true;
            this.server.on("connection", c => {
                this.connections.add(c);
                c.on("close", () => {
                    this.connections.delete(c);
                });
            });
        }
    }
    /**
     * Close the server (if listening).
     */
    close(cb) {
        this.server.close(cb);
        for (const connection of this.connections.values()) {
            connection.destroy();
        }
    }
    /**
     * Resolve a URL to the server location.
     */
    url(path) {
        const addr = this.server.address();
        if (!addr) {
            throw new TypeError("Server is not listening");
        }
        const origin = typeof addr === "string"
            ? addr
            : `${this.protocol}://127.0.0.1:${addr.port}`;
        return path ? url_1.resolve(origin, path) : origin;
    }
}
exports.ServerAddress = ServerAddress;
//# sourceMappingURL=index.js.map